@@ -96,12 +96,14 @@
 // Hashtable class, used to implement the hashed associative containers
 // hash_set and hash_map.
 
-#include <google/sparsehash/config.h>
+#include <google/sparsehash/sparseconfig.h>
 #include <assert.h>
 #include <stdlib.h>             // for abort()
 #include <algorithm>            // For swap(), eg
 #include <iostream>             // For cerr
 #include <memory>               // For uninitialized_fill, uninitialized_copy
+#include <utility>              // for pair<>
+#include <iterator>             // for facts about iterator tags
 #include <google/type_traits.h> // for true_type, integral_constant, etc.
 
 _START_GOOGLE_NAMESPACE_
@@ -122,13 +124,10 @@ struct dense_hashtable_const_iterator;
 template <class V, class K, class HF, class ExK, class EqK, class A>
 struct dense_hashtable_iterator {
  public:
-  typedef dense_hashtable<V,K,HF,ExK,EqK,A>                dense_hashtable;
   typedef dense_hashtable_iterator<V,K,HF,ExK,EqK,A>       iterator;
   typedef dense_hashtable_const_iterator<V,K,HF,ExK,EqK,A> const_iterator;
 
-#ifdef UNDERSTANDS_ITERATOR_TAGS
   typedef STL_NAMESPACE::forward_iterator_tag iterator_category;
-#endif
   typedef V value_type;
   typedef ptrdiff_t difference_type;
   typedef size_t size_type;
@@ -136,7 +135,7 @@ struct dense_hashtable_iterator {
   typedef V* pointer;
 
   // "Real" constructor and default constructor
-  dense_hashtable_iterator(const dense_hashtable *h,
+  dense_hashtable_iterator(const dense_hashtable<V,K,HF,ExK,EqK,A> *h,
                            pointer it, pointer it_end, bool advance)
     : ht(h), pos(it), end(it_end)   {
     if (advance)  advance_past_empty_and_deleted();
@@ -166,7 +165,7 @@ struct dense_hashtable_iterator {
 
 
   // The actual data
-  const dense_hashtable *ht;
+  const dense_hashtable<V,K,HF,ExK,EqK,A> *ht;
   pointer pos, end;
 };
 
@@ -175,13 +174,10 @@ struct dense_hashtable_iterator {
 template <class V, class K, class HF, class ExK, class EqK, class A>
 struct dense_hashtable_const_iterator {
  public:
-  typedef dense_hashtable<V,K,HF,ExK,EqK,A>                dense_hashtable;
   typedef dense_hashtable_iterator<V,K,HF,ExK,EqK,A>       iterator;
   typedef dense_hashtable_const_iterator<V,K,HF,ExK,EqK,A> const_iterator;
 
-#ifdef UNDERSTANDS_ITERATOR_TAGS
   typedef STL_NAMESPACE::forward_iterator_tag iterator_category;
-#endif
   typedef V value_type;
   typedef ptrdiff_t difference_type;
   typedef size_t size_type;
@@ -189,7 +185,7 @@ struct dense_hashtable_const_iterator {
   typedef const V* pointer;
 
   // "Real" constructor and default constructor
-  dense_hashtable_const_iterator(const dense_hashtable *h,
+  dense_hashtable_const_iterator(const dense_hashtable<V,K,HF,ExK,EqK,A> *h,
                                  pointer it, pointer it_end, bool advance)
     : ht(h), pos(it), end(it_end)   {
     if (advance)  advance_past_empty_and_deleted();
@@ -222,7 +218,7 @@ struct dense_hashtable_const_iterator {
 
 
   // The actual data
-  const dense_hashtable *ht;
+  const dense_hashtable<V,K,HF,ExK,EqK,A> *ht;
   pointer pos, end;
 };
 
@@ -254,13 +250,14 @@ class dense_hashtable {
   static const float HT_OCCUPANCY_FLT; // = 0.8;
 
   // How empty we let the table get before we resize lower.
+  // (0.0 means never resize lower.)
   // It should be less than OCCUPANCY_FLT / 2 or we thrash resizing
   static const float HT_EMPTY_FLT; // = 0.4 * HT_OCCUPANCY_FLT
 
   // Minimum size we're willing to let hashtables be.
   // Must be a power of two, and at least 4.
-  // Note, however, that for a given hashtable, the minimum size is
-  // determined by the first constructor arg, and may be >HT_MIN_BUCKETS.
+  // Note, however, that for a given hashtable, the initial size is a
+  // function of the first constructor arg, and may be >HT_MIN_BUCKETS.
   static const size_t HT_MIN_BUCKETS  = 32;
 
 
@@ -310,6 +307,9 @@ class dense_hashtable {
 
  public:
   void set_deleted_key(const value_type &val) {
+    // the empty indicator (if specified) and the deleted indicator
+    // must be different
+    assert(!use_empty || !equals(get_key(val), get_key(emptyval)));
     // It's only safe to change what "deleted" means if we purge deleted guys
     squash_deleted();
     use_deleted = true;
@@ -396,6 +396,9 @@ class dense_hashtable {
   void set_empty_key(const value_type &val) {
     // Once you set the empty key, you can't change it
     assert(!use_empty);
+    // The deleted indicator (if specified) and the empty indicator
+    // must be different.
+    assert(!use_deleted || !equals(get_key(val), get_key(delval)));
     use_empty = true;
     set_value(&emptyval, val);
 
@@ -436,11 +439,11 @@ class dense_hashtable {
     assert((bucket_count() & (bucket_count()-1)) == 0); // is a power of two
     assert(bucket_count() >= HT_MIN_BUCKETS);
 
-    if ( (num_elements-num_deleted) <= shrink_threshold &&
+    if ( (num_elements-num_deleted) < shrink_threshold &&
          bucket_count() > HT_MIN_BUCKETS ) {
       size_type sz = bucket_count() / 2;    // find how much we should shrink
       while ( sz > HT_MIN_BUCKETS &&
-              (num_elements - num_deleted) <= sz * HT_EMPTY_FLT )
+              (num_elements - num_deleted) < sz * HT_EMPTY_FLT )
         sz /= 2;                            // stay a power of 2
       dense_hashtable tmp(*this, sz);       // Do the actual resizing
       swap(tmp);                            // now we are tmp
@@ -465,8 +468,12 @@ class dense_hashtable {
     }
   }
 
-  // Increase number of buckets, assuming value_type has trivial
-  // copy constructor and assignment operator
+  // Increase number of buckets, assuming value_type has trivial copy
+  // constructor and destructor.  (Really, we want it to have "trivial
+  // move", because that's what realloc does.  But there's no way to
+  // capture that using type_traits, so we pretend that move(x, y) is
+  // equivalent to "x.~T(); new(x) T(y);" which is pretty much
+  // correct, if a bit conservative.)
   void expand_array(size_t resize_to, true_type) {
     table = (value_type *) realloc(table, resize_to * sizeof(value_type));
     assert(table);
@@ -495,10 +502,10 @@ class dense_hashtable {
     const size_type resize_to = min_size(ht.size(), min_buckets_wanted);
     if ( resize_to > bucket_count() ) { // we don't have enough buckets
       typedef integral_constant<bool,
-                                (has_trivial_copy<value_type>::value &&
-                                 has_trivial_assign<value_type>::value)>
-        is_simple_type;
-      expand_array(resize_to, is_simple_type());
+          (has_trivial_copy<value_type>::value &&
+           has_trivial_destructor<value_type>::value)>
+          realloc_ok; // we pretend mv(x,y) == "x.~T(); new(x) T(y)"
+      expand_array(resize_to, realloc_ok());
       num_buckets = resize_to;
       reset_thresholds();
     }
@@ -623,6 +630,18 @@ class dense_hashtable {
     num_deleted = 0;
   }
 
+  // Clear the table without resizing it.
+  // Mimicks the stl_hashtable's behaviour when clear()-ing in that it
+  // does not modify the bucket count
+  void clear_no_resize() {
+    if (table) {
+      set_empty(0, num_buckets);
+    }
+    // don't consider to shrink before another erase()
+    reset_thresholds();
+    num_elements = 0;
+    num_deleted = 0;
+  }
 
   // LOOKUP ROUTINES
  private:
@@ -750,6 +769,7 @@ class dense_hashtable {
       insert(*f);
   }
 
+
   // DELETION ROUTINES
   size_type erase(const key_type& key) {
     const_iterator pos = find(key);   // shrug: shouldn't need to be const
@@ -786,9 +806,21 @@ class dense_hashtable {
 
   // COMPARISON
   bool operator==(const dense_hashtable& ht) const {
-    // We really want to check that the hash functions are the same
-    // but alas there's no way to do this.  We just hope.
-    return ( num_deleted == ht.num_deleted && table == ht.table );
+    if (size() != ht.size()) {
+      return false;
+    } else if (this == &ht) {
+      return true;
+    } else {
+      // Iterate through the elements in "this" and see if the
+      // corresponding element is in ht
+      for ( const_iterator it = begin(); it != end(); ++it ) {
+        const_iterator it2 = ht.find(get_key(*it));
+        if ((it2 == ht.end()) || (*it != *it2)) {
+          return false;
+        }
+      }
+      return true;
+    }
   }
   bool operator!=(const dense_hashtable& ht) const {
     return !(*this == ht);
@@ -891,7 +923,7 @@ const float dense_hashtable<V,K,HF,ExK,EqK,A>::HT_OCCUPANCY_FLT = 0.5f;
 // How empty we let the table get before we resize lower.
 // It should be less than OCCUPANCY_FLT / 2 or we thrash resizing
 template <class V, class K, class HF, class ExK, class EqK, class A>
-const float dense_hashtable<V,K,HF,ExK,EqK,A>::HT_EMPTY_FLT = 0.4 *
+const float dense_hashtable<V,K,HF,ExK,EqK,A>::HT_EMPTY_FLT = 0.4f *
 dense_hashtable<V,K,HF,ExK,EqK,A>::HT_OCCUPANCY_FLT;
 
 _END_GOOGLE_NAMESPACE_
