@@ -105,9 +105,11 @@
 // Hashtable class, used to implement the hashed associative containers
 // hash_set and hash_map.
 
-#include <google/sparsehash/config.h>
+#include <google/sparsehash/sparseconfig.h>
 #include <assert.h>
 #include <algorithm>              // For swap(), eg
+#include <iterator>               // for facts about iterator tags
+#include <utility>                // for pair<>
 #include <google/sparsetable>     // Since that's basically what we are
 
 _START_GOOGLE_NAMESPACE_
@@ -133,14 +135,11 @@ struct sparse_hashtable_const_iterator;
 template <class V, class K, class HF, class ExK, class EqK, class A>
 struct sparse_hashtable_iterator {
  public:
-  typedef sparse_hashtable<V,K,HF,ExK,EqK,A>                sparse_hashtable;
   typedef sparse_hashtable_iterator<V,K,HF,ExK,EqK,A>       iterator;
   typedef sparse_hashtable_const_iterator<V,K,HF,ExK,EqK,A> const_iterator;
   typedef typename sparsetable<V>::nonempty_iterator        st_iterator;
 
-#ifdef UNDERSTANDS_ITERATOR_TAGS
   typedef STL_NAMESPACE::forward_iterator_tag iterator_category;
-#endif
   typedef V value_type;
   typedef ptrdiff_t difference_type;
   typedef size_t size_type;
@@ -148,7 +147,7 @@ struct sparse_hashtable_iterator {
   typedef V* pointer;
 
   // "Real" constructor and default constructor
-  sparse_hashtable_iterator(const sparse_hashtable *h,
+  sparse_hashtable_iterator(const sparse_hashtable<V,K,HF,ExK,EqK,A> *h,
                             st_iterator it, st_iterator it_end)
     : ht(h), pos(it), end(it_end)   { advance_past_deleted(); }
   sparse_hashtable_iterator() { }      // not ever used internally
@@ -176,7 +175,7 @@ struct sparse_hashtable_iterator {
 
 
   // The actual data
-  const sparse_hashtable *ht;
+  const sparse_hashtable<V,K,HF,ExK,EqK,A> *ht;
   st_iterator pos, end;
 };
 
@@ -184,14 +183,11 @@ struct sparse_hashtable_iterator {
 template <class V, class K, class HF, class ExK, class EqK, class A>
 struct sparse_hashtable_const_iterator {
  public:
-  typedef sparse_hashtable<V,K,HF,ExK,EqK,A>                sparse_hashtable;
   typedef sparse_hashtable_iterator<V,K,HF,ExK,EqK,A>       iterator;
   typedef sparse_hashtable_const_iterator<V,K,HF,ExK,EqK,A> const_iterator;
   typedef typename sparsetable<V>::const_nonempty_iterator  st_iterator;
 
-#ifdef UNDERSTANDS_ITERATOR_TAGS
   typedef STL_NAMESPACE::forward_iterator_tag iterator_category;
-#endif
   typedef V value_type;
   typedef ptrdiff_t difference_type;
   typedef size_t size_type;
@@ -199,7 +195,7 @@ struct sparse_hashtable_const_iterator {
   typedef const V* pointer;
 
   // "Real" constructor and default constructor
-  sparse_hashtable_const_iterator(const sparse_hashtable *h,
+  sparse_hashtable_const_iterator(const sparse_hashtable<V,K,HF,ExK,EqK,A> *h,
                                   st_iterator it, st_iterator it_end)
     : ht(h), pos(it), end(it_end)   { advance_past_deleted(); }
   // This lets us convert regular iterators to const iterators
@@ -230,7 +226,7 @@ struct sparse_hashtable_const_iterator {
 
 
   // The actual data
-  const sparse_hashtable *ht;
+  const sparse_hashtable<V,K,HF,ExK,EqK,A> *ht;
   st_iterator pos, end;
 };
 
@@ -238,13 +234,10 @@ struct sparse_hashtable_const_iterator {
 template <class V, class K, class HF, class ExK, class EqK, class A>
 struct sparse_hashtable_destructive_iterator {
  public:
-  typedef sparse_hashtable<V,K,HF,ExK,EqK,A>                sparse_hashtable;
   typedef sparse_hashtable_destructive_iterator<V,K,HF,ExK,EqK,A> iterator;
   typedef typename sparsetable<V>::destructive_iterator     st_iterator;
 
-#ifdef UNDERSTANDS_ITERATOR_TAGS
   typedef STL_NAMESPACE::forward_iterator_tag iterator_category;
-#endif
   typedef V value_type;
   typedef ptrdiff_t difference_type;
   typedef size_t size_type;
@@ -252,7 +245,8 @@ struct sparse_hashtable_destructive_iterator {
   typedef V* pointer;
 
   // "Real" constructor and default constructor
-  sparse_hashtable_destructive_iterator(const sparse_hashtable *h,
+  sparse_hashtable_destructive_iterator(const
+                                        sparse_hashtable<V,K,HF,ExK,EqK,A> *h,
                                         st_iterator it, st_iterator it_end)
     : ht(h), pos(it), end(it_end)   { advance_past_deleted(); }
   sparse_hashtable_destructive_iterator() { }          // never used internally
@@ -280,7 +274,7 @@ struct sparse_hashtable_destructive_iterator {
 
 
   // The actual data
-  const sparse_hashtable *ht;
+  const sparse_hashtable<V,K,HF,ExK,EqK,A> *ht;
   st_iterator pos, end;
 };
 
@@ -742,7 +736,6 @@ class sparse_hashtable {
     return insert_noresize(obj);
   }
 
-#ifdef UNDERSTANDS_ITERATOR_TAGS
   // When inserting a lot at a time, we specialize on the type of iterator
   template <class InputIterator>
   void insert(InputIterator f, InputIterator l) {
@@ -767,13 +760,6 @@ class sparse_hashtable {
     for ( ; f != l; ++f)
       insert(*f);
   }
-#else
-  template <class InputIterator>
-  void insert(InputIterator f, InputIterator l) {
-    for ( ; f != l; ++f)
-      insert(*f);
-  }
-#endif
 
 
   // DELETION ROUTINES
@@ -890,7 +876,7 @@ const float sparse_hashtable<V,K,HF,ExK,EqK,A>::HT_OCCUPANCY_FLT = 0.8f;
 // How empty we let the table get before we resize lower.
 // It should be less than OCCUPANCY_FLT / 2 or we thrash resizing
 template <class V, class K, class HF, class ExK, class EqK, class A>
-const float sparse_hashtable<V,K,HF,ExK,EqK,A>::HT_EMPTY_FLT = 0.4 *
+const float sparse_hashtable<V,K,HF,ExK,EqK,A>::HT_EMPTY_FLT = 0.4f *
 sparse_hashtable<V,K,HF,ExK,EqK,A>::HT_OCCUPANCY_FLT;
 
 _END_GOOGLE_NAMESPACE_
